
#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
#endregion

namespace BasicEditor.Classes
{
    /// <summary>
    /// This is a game component that implements IUpdateable.
    /// </summary>
    public class GUIWidget : xFrippery.Control, Interfaces.IGUIWidget
    {
        private Texture2D tex = null;
        private Rectangle sourceRectangle = Rectangle.Empty;
        private Color color = Color.White;
        private string textureName = string.Empty;
        private bool textureNeedsUpdate = true;


        public virtual void Initilize()
        {
        }

        public Color Color
        {
            get { return color; }
            set { color = value; }
        }     

        public Rectangle SourceRectangle
        {
            get { return sourceRectangle; }
            set { sourceRectangle = value; }
        }

        public Texture2D Texture
        {
            get { return tex; }
            set { tex = value; }
        }

        public string TextureName
        {
            get { return textureName; }
            set { 
                textureName = value;
            textureNeedsUpdate = true;
        }
        }
        
        public GUIWidget() : base()
        {
        }

        /// <summary>
        /// Allows the game component to update itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public virtual void Update(Editor game, GameTime gameTime)
        {
            // TODO: Add your update code here
            if (textureNeedsUpdate && game.guiTextures.ContainsKey(textureName))
            {
                this.tex = game.guiTextures[textureName];
                textureNeedsUpdate = false;
            }

            // update any children  
            foreach (Interfaces.IGUIWidget child in this.children)
            {
                //if (child.GetType() == typeof(Interfaces.IGUIWidget)) { 
                //Interfaces.IGUIWidget gw = (Interfaces.IGUIWidget)child;
                //gw.Update(game, gameTime);
                child.Update(game, gameTime);
                  //         }
 }
        }

        /// <summary>
        /// Allows the game component to draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public virtual void Draw(GameTime gameTime, SpriteBatch sb)
        {
            this.Draw(this, gameTime, sb);
        }

        protected virtual void Draw(Interfaces.IGUIWidget c, GameTime gameTime, SpriteBatch sb)
        {
            if (!this.Visible) { return; }

            if (this.tex != null)
            {
                // calc dest rect
                Vector2 v = new Vector2(this.Left, this.Top);
                if (this.Parent != null)
                {
                    v = this.Parent.PointToScreen(new Vector2(this.Left, this.Top));
                }
                Rectangle dr = new Rectangle((int)v.X, (int)v.Y, this.Width, this.Height);
               
                // draw control
                sb.Draw(this.Texture, dr, this.sourceRectangle, this.color);
            }

            // draw any children  
            foreach (Interfaces.IGUIWidget child in this.children)
            {
                //Classes.GUIWidget gw = (GUIWidget)child;
                //gw.Draw(gameTime, sb);
                child.Draw(gameTime, sb);
            }
        }
             

       
    }
}


