
#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Input;

#endregion

namespace BasicEditor.Components
{
    /// <summary>
    /// This is a game component that implements IUpdateable.
    /// </summary>
    public partial class BlockEditor : Microsoft.Xna.Framework.GameComponent
    {

        protected Editor editor;
        internal protected string firstTexture=string.Empty;
        internal protected string secondTexture=string.Empty;
        private BlockType firstBlockType = BlockType.Platform;
        private BlockType secondBlockType = BlockType.Platform;
        
        public BlockType SecondBlockType
        {
            get { return secondBlockType; }
            set { secondBlockType = value; }
        }


        public BlockType FirstBlockType
        {
            get { return firstBlockType; }
            set { firstBlockType = value; }
        }

        public string FirstTexture
        {
            get { return firstTexture; }
            set { firstTexture = value; }
        }

        public string SecondTexture
        {
            get { return secondTexture; }
            set { secondTexture = value; }
        }

        public BlockEditor(Game game)
            : base(game)
        {
            // TODO: Construct any child components here
            this.editor = (Editor)game;
        }


        /// <summary>
        /// Allows the game component to perform any initialization it needs to before starting
        /// to run.  This is where it can query for any required services and load content.
        /// </summary>
        public override void Initialize()
        {
            // TODO: Add your initialization code here

            base.Initialize();
        }


        /// <summary>
        /// Allows the game component to update itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Update(GameTime gameTime)
        {
            // update pointer pos
            MouseState state = Mouse.GetState();
            editor.PointerPosition = new Vector2(state.X, state.Y);

            editor.GUI.
            // check if drawing with left mouse button
            if (state.LeftButton == ButtonState.Pressed)
            {
                DrawBlock(firstBlockType, firstTexture);
            }
                   
            // check if drawing with left mouse button
            if (state.RightButton == ButtonState.Pressed)
            {
                DrawBlock(secondBlockType, secondTexture);
            }

            base.Update(gameTime);
        }

        private void DrawBlock(BlockType type, string t)
        {
            // calc block position
            int x = (int)((this.editor.pointerPosition.X + this.editor.cam.Position.X) / this.editor.CurrentMap.blockWidth);
            int y = (int)((this.editor.pointerPosition.Y + this.editor.cam.Position.Y) / this.editor.CurrentMap.blockHeight);
            // calc source pos
            Rectangle r = new Rectangle(0, 0, (int)this.editor.CurrentMap.blockWidth, (int)this.editor.CurrentMap.blockHeight);
            // create the block object if texture is assigned
            Block b = null;
            if (firstTexture != string.Empty) { b = Helpers.CreateBlock(type, r, this.editor.gameTextures[t]); }
            // set the block within the map
            this.editor.CurrentMap.SetBlock(x, y, b);
        }


    }
}


