
#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
using System.Reflection;
#endregion

namespace BasicEditor
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Editor : Microsoft.Xna.Framework.Game
    {
        internal GraphicsDeviceManager graphics;
        internal ContentManager content;
        internal protected Map map;
        internal protected Vector2 pointerPosition;
        internal protected Vector2 pointerHotSpot;
        internal protected Collections.TextureDictionary gameTextures = new BasicEditor.Collections.TextureDictionary();
        internal protected Collections.TextureDictionary guiTextures = new BasicEditor.Collections.TextureDictionary();
        internal protected Texture2D pointerTexture;
        internal protected SpriteBatch sb;
        internal protected XNAFont font;
        internal protected string dialogTexture = "Dialog";
        internal protected string fontTexture = "Simple";
        internal protected string pointerName = "StdPointer";
        internal protected string whiteTexture = "White";
        internal protected string gameTitle = string.Empty;
        internal protected Camera cam;
        internal protected ButtonState pointerState = ButtonState.Released;
        private Collections.CameraDictionary cameras = new BasicEditor.Collections.CameraDictionary();
        private Axiom.Controllers.ControllerManager controllerManager;
        internal protected Components.GUIRenderer gui;
        internal protected xFrippery.xFrippery guiInputHandler;


        public Components.GUIRenderer GUI
        {
            get { return this.gui; }
        }
	
        public Axiom.Controllers.ControllerManager ControllerManager
        {
            get { return controllerManager; }
            // set { controllerManager = value; }
        }

        public Collections.CameraDictionary Cameras
        {
            get { return cameras = new BasicEditor.Collections.CameraDictionary(); }
        }

        public ButtonState PointerState
        {
            get { return pointerState; }
            set { pointerState = value; }
        }

        public string GameTitle
        {
            get { return gameTitle; }
            set { gameTitle = value; }
        }

        public GraphicsDevice GraphicsDevice
        {
            get { return this.graphics.GraphicsDevice; }
        }

        public Map CurrentMap
        {
            get { return map; }
        }

        public Texture2D PointerTexture
        {
            get { return pointerTexture; }
            set { pointerTexture = value; }
        }

        public string PointerName
        {
            get { return pointerName; }
            set { pointerName = value; }
        }

        public Vector2 PointerHotSpot
        {
            get { return pointerPosition; }
            set { pointerPosition = value; }
        }

        public Vector2 PointerPosition
        {
            get { return pointerPosition; }
            set { pointerPosition = value; }
        }

        public float PointerPositionX
        {
            get { return pointerPosition.X; }
            set { pointerPosition.X = value; }
        }

        public float PointerPositionY
        {
            get { return pointerPosition.Y; }
            set { pointerPosition.Y = value; }
        }

        public Editor()
        {
            graphics = new GraphicsDeviceManager(this);
            content = new ContentManager(Services);
        }


        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here
            this.controllerManager = new Axiom.Controllers.ControllerManager(this);
            guiInputHandler = new xFrippery.xFrippery();

            map = new Map();
            //map.Resize(6, 6);
            map.Initilize();

            sb = new SpriteBatch(this.graphics.GraphicsDevice);
            gui = new BasicEditor.Components.GUIRenderer(this);
            this.Components.Add(gui);

            // add a white gui texture
            Texture2D wt = new Texture2D(this.GraphicsDevice, 4, 4, 1, ResourceUsage.Dynamic, SurfaceFormat.Color, ResourceManagementMode.Manual);
            byte[] pdata = new byte[4 * 4 * 4];
            for (int idx = 0; idx <= pdata.Length - 1; idx++)
            {
                pdata[idx] = byte.MaxValue;
            }
            wt.SetData<byte>(pdata);
            this.guiTextures.Add(this.whiteTexture, wt);

            // add map loader component to load a map from command line args
            Components.MapLoader ml = new Components.MapLoader(this, GetArgs());
            ml.FadeOut = false; // will not slowly fade out the loading text
            this.Components.Add(ml);

            try
            {
                Assembly assembly = Assembly.GetExecutingAssembly();
                string file = "BasicEditor.Resources.Simple.DDS.xml";
                font = XNAFont.LoadFont(this.graphics.GraphicsDevice, assembly.GetManifestResourceStream(file), string.Empty);
            }
            catch (Exception ex)
            {
                System.Diagnostics.Debug.Print(ex.ToString());
                this.Exit();
                Environment.Exit(Helpers.EXITCODE_BadDefaultFont);
                return;
            }

            cam = new Camera();
            cam.Viewport = this.GraphicsDevice.Viewport;
            cam.ProjectionType = Projection.Orthographic;
            cam.Position = Vector3.Backward;// *(cam.Viewport.Width / 2);
            // cam.Position += Vector3.Right * 16;
            //cam.Position += Vector3.Right * (cam.Viewport.Width / 2);
            //cam.Direction += Vector3.Right * (cam.Viewport.Width / 2);
            //cam.Position += Vector3.Up * (cam.Viewport.Height / 2);
            //cam.Direction += Vector3.Up * (cam.Viewport.Height / 2);

            // cam.AutoAspectRatio = false;
            // cam.AspectRatio = 1;
            // cam.Orientation = Vector3.Up;

            // cam.Position = Vector3.Backward * 100;// this.GraphicsDevice.Viewport.Width / 2.0f;
            base.Initialize();
        }

        private void LoadDesktopGUI()
        {
            // attempt to create widget
            Classes.Widgets.ButtonTexturesWidget w;
            w = Helpers.CreateWidget<Classes.Widgets.ButtonTexturesWidget>("BasicEditor.Resources.ButtonTexturesWidget.xml",
                                                                           "root");
            // add to desktop
            this.gui.Desktop.Add("ButtonTextures", w);

            //NOTE: Setting bounds directly at this point will not position the widget at the top right corner
            //      begause the width "(w.Width + 4)" of the widget has yet to be set. So the fallowing line
            //      of code will not work
            //w.Bounds = new Rectangle(cam.Viewport.Width - (w.Width + 4), 4, 128, 3);
            
            // position widiget at top right corner
            w.Width = 128;
            w.Height = 64;
            w.Left = cam.Viewport.Width - (w.Width + 4);
            w.Top = 4;
        }

        private string GetArgs()
        {
            string[] tmp = Environment.GetCommandLineArgs();
            return string.Join(" ", tmp, 1, tmp.Length - 1);
        }


        /// <summary>
        /// Load your graphics content.  If loadAllContent is true, you should
        /// load content from both ResourceManagementMode pools.  Otherwise, just
        /// load ResourceManagementMode.Manual content.
        /// </summary>
        /// <param name="loadAllContent">Which type of content to load.</param>
        protected override void LoadGraphicsContent(bool loadAllContent)
        {
            if (loadAllContent)
            {
                // TODO: Load any ResourceManagementMode.Automatic content
                Assembly assembly = Assembly.GetExecutingAssembly();
                string file = "BasicEditor.Resources.Dialog.png";
                Texture2D tex = Texture2D.FromFile(this.GraphicsDevice, assembly.GetManifestResourceStream(file));
                this.guiTextures.Add(this.dialogTexture, tex);

                file = "BasicEditor.Resources.StdPointer.png";
                tex = Texture2D.FromFile(this.GraphicsDevice, assembly.GetManifestResourceStream(file));
                this.guiTextures.Add(this.pointerName, tex);
                this.pointerTexture = tex;

                file = "BasicEditor.Resources.Simple.DDS";
                tex = Texture2D.FromFile(this.GraphicsDevice, assembly.GetManifestResourceStream(file));
                this.guiTextures.Add(this.fontTexture, tex);
                this.font.Texture = tex;

                LoadDesktopGUI();
            }

            // TODO: Load any ResourceManagementMode.Manual content
        }


        /// <summary>
        /// Unload your graphics content.  If unloadAllContent is true, you should
        /// unload content from both ResourceManagementMode pools.  Otherwise, just
        /// unload ResourceManagementMode.Manual content.  Manual content will get
        /// Disposed by the GraphicsDevice during a Reset.
        /// </summary>
        /// <param name="unloadAllContent">Which type of content to unload.</param>
        protected override void UnloadGraphicsContent(bool unloadAllContent)
        {
            if (unloadAllContent == true)
            {
                content.Unload();
            }
        }


        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the default game to exit on Xbox 360 and Windows
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            // TODO: Add your update logic here
            if (cam.NeedsUpdate) cam.Update();
            if (cam.ViewportChanged)
            {
                this.GraphicsDevice.Viewport = cam.Viewport;
                cam.ViewportChanged = false;
            }

            base.Update(gameTime);
        }


        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            graphics.GraphicsDevice.Clear(Color.CornflowerBlue);

            base.Draw(gameTime);

            // Draw Pointer
            sb.Begin();
            sb.Draw(this.pointerTexture, this.pointerPosition - this.pointerHotSpot, Color.White);
            sb.End();

        }
    }
}