#define DotNetStyleEvents
//using CommonEventArgs;
using System;
using Microsoft;
//using System.Windows.Forms;
using System.Collections;
//using System.Drawing;
//using Microsoft.VisualBasic;
//using System.Windows;
//using System.Diagnostics;
//using System.Data;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

//Created by: X
//Web: http://www.createdbyx.com/



//Public Enum ControlUnits As Integer
//    Pixel = 0
//    Percent = 1
//End Enum

namespace xFrippery
{
    public class Control : IControl
    {
        protected internal bool visible = true;
        protected internal IControl parent = null;
        protected internal ControlCollection children = new ControlCollection(this);
        protected internal string name = string.Empty;
        protected internal string data = string.Empty;
        protected internal Rectangle bounds;
        protected internal AnchorStyles anchor = AnchorStyles.Top | AnchorStyles.Left;


        public Control()
        {
        }

        public virtual IControl GetControlFromPoint(float X, float Y)
        {
            if (this.children == null) { return null; }

            Vector2 TL;
            Vector2 BR;
            TL = this.PointToScreen(new Vector2());
            BR = this.PointToScreen(new Vector2(this.bounds.Width, this.bounds.Height));

            for (int idx = children.Count - 1; idx >= 0; idx--)
            {
                IControl Tmp;

                Tmp = children[idx].GetControlFromPoint(X, Y);
                if (Tmp != null)
                {
                    return Tmp;
                }
            }

            if (X >= TL.X && Y >= TL.Y && X <= BR.X && Y <= BR.Y)
            {
                return this;
            }

            return null;
        }

        public virtual AnchorStyles Anchor
        {
            get { return anchor; }
            set { anchor = value; }
        }

        public virtual int Height
        {
            get { return bounds.Height; }
            set
            {
                int old = bounds.Height;
                bounds.Height = value;
                if (this.HeightChanged != null) { this.HeightChanged(this, value, old); }
            }
        }

        public virtual int Width
        {
            get { return bounds.Width; }
            set
            {
                int old = bounds.Width;
                bounds.Width = value;
                  if (this.WidthChanged != null) { this.WidthChanged(this, value, old); }
            }
        }

        public virtual int Top
        {
            get { return bounds.Y; }
            set { bounds.Y = value; }
        }

        public virtual int Left
        {
            get { return bounds.X; }
            set { bounds.X = value; }
        }

        public virtual Rectangle Bounds
        {
            get { return bounds; }
            set
            {
                Rectangle old = bounds;
                bounds = value;
                if (this.BoundsChanged != null) { this.BoundsChanged(this, value, old); }
            }
        }

        public virtual Vector2 PointToScreen(Vector2 Pt)
        {
            IControl P = this.parent;
            Vector2 Tmp = new Vector2(this.bounds.Left + Pt.X, this.bounds.Top + Pt.Y);
            while (true)
            {
                if (P != null)
                {
                    Tmp.X += P.Bounds.Left;
                    Tmp.Y += P.Bounds.Top;
                    P = P.Parent;
                }
                else
                {
                    break;
                }
            }
            return Tmp;
        }

        public virtual Vector2 PointToClient(Vector2 Pt)
        {
            Vector2 Tmp = this.PointToScreen(new Vector2());
            Tmp.X = Pt.X - Tmp.X;
            Tmp.Y = Pt.Y - Tmp.Y;
            return Tmp;
        }

        public virtual void MoveToFront()
        {
            if (parent != null && parent.Children != null)
            {
                parent.Children.MoveToFront(this);
            }
        }

        public virtual void MoveToBack()
        {
            if (parent != null && parent.Children != null)
            {
                parent.Children.MoveToBack(this);
            }
        }

        public virtual string Data
        {
            get { return data; }
            set { data = value; }
        }

        public virtual string Name
        {
            get { return name; }
            set { name = value; }
        }

        public virtual void DoMouseDown(object sender, MouseButtons b, Vector2 pos) { }
        public virtual void DoMouseMove(object sender, MouseButtons b, Vector2 pos) { }
        public virtual void DoMouseUp(object sender, MouseButtons b, Vector2 pos) { }
        public virtual void DoClick(object sender, MouseButtons b, Vector2 pos) { }


        public virtual ControlCollection Children
        {
            get { return children; }
            set { children = value; }
        }

        public virtual IControl Parent
        {
            get { return parent; }
            set
            {
                // assign new value
                if (this.parent != null)
                {
                    this.parent.WidthChanged -= new WidthEventHandler(this.Parent_WidthChange);
                    this.parent.HeightChanged -= new HeightEventHandler(this.Parent_HeightChanged);
                    this.parent.BoundsChanged -= new BoundsEventHandler(this.Parent_BoundsChanged);
                }
                parent = value;
                if (this.parent != null)
                {
                    this.parent.WidthChanged += new WidthEventHandler(this.Parent_WidthChange);
                    this.parent.HeightChanged += new HeightEventHandler(this.Parent_HeightChanged);
                    this.parent.BoundsChanged += new BoundsEventHandler(this.Parent_BoundsChanged);
                }
            }
        }

        public virtual bool Visible
        {
            get { return visible; }
            set { visible = value; }
        }

        private void Parent_BoundsChanged(object sender, Rectangle value, Rectangle oldvalue)
        {
                ApplyAnchor(value.Width - oldvalue.Width, value.Height - oldvalue.Height);
        }

        private void Parent_HeightChanged(object sender, int value, int oldvalue)
        {
                ApplyAnchor(0, value - oldvalue);
        }

        private void Parent_WidthChange(object sender, int value, int oldvalue)
        {
                ApplyAnchor(value - oldvalue, 0);
        }

        private void ApplyAnchor(int WidthDiff, int HeightDiff)
        {
            // no anchor
            if (this.anchor == AnchorStyles.None)
            {
                this.Left += WidthDiff;
                this.Top += HeightDiff;
                return;
            }

            // width
            if (((AnchorStyles.Right & this.anchor) == AnchorStyles.Right) &&
                ((AnchorStyles.Left & this.anchor) == AnchorStyles.Left))
            {
                this.bounds.Width += WidthDiff;
            }
            else if (!((AnchorStyles.Left & this.anchor) == AnchorStyles.Left) &&
                     !((AnchorStyles.Right & this.anchor) == AnchorStyles.Right))
            {
                this.bounds.X += WidthDiff;

            }
            else if ((AnchorStyles.Right & this.anchor) == AnchorStyles.Right)
            {
                this.bounds.X += WidthDiff;
            }

            // height
            if (((AnchorStyles.Bottom & this.anchor) == AnchorStyles.Bottom) &&
                ((AnchorStyles.Top & this.anchor) == AnchorStyles.Top))
            {
                this.bounds.Height += HeightDiff;

            }
            else if (!((AnchorStyles.Top & this.anchor) == AnchorStyles.Top) &&
                     !((AnchorStyles.Bottom & this.anchor) == AnchorStyles.Bottom))
            {
                this.bounds.Y += HeightDiff;

            }
            else if ((AnchorStyles.Bottom & this.anchor) == AnchorStyles.Bottom)
            {
                this.bounds.Y += HeightDiff; 
            }
        }

        public virtual object Clone()
        {
            return this.Clone(false);
        }

        public virtual object Clone(bool CloneChildren)
        {
            Control Tmp = new Control();
            Tmp.anchor = this.anchor;
            Tmp.bounds = this.bounds;
            if (CloneChildren)
            {
                Tmp.Children.Add(this.Children);
            }

            Tmp.data = this.data;
            Tmp.name = this.name;
            Tmp.parent = this.parent;
            return Tmp;
        }


        #region IControl Members

        public event ClickEventHandler Click;
        public event MouseDownEventHandler MouseDown;
        public event MouseMoveEventHandler MouseMove;
        public event MouseUpEventHandler MouseUp;
        public event KeyUpEventHandler KeyUp;
        public event KeyDownEventHandler KeyDown;
        public event KeyClickEventHandler KeyClick;
        public event WidthEventHandler WidthChanged;
        public event HeightEventHandler HeightChanged;
        public event BoundsEventHandler BoundsChanged;


        event ClickEventHandler IControl.Click
        {
            add { this.Click += value; }
            remove { this.Click -= value; }
        }

        event MouseDownEventHandler IControl.MouseDown
        {
            add { this.MouseDown += value; }
            remove { this.MouseDown -= value; }
        }

        event MouseMoveEventHandler IControl.MouseMove
        {
            add { this.MouseMove += value; }
            remove { this.MouseMove -= value; }
        }

        event MouseUpEventHandler IControl.MouseUp
        {
            add { this.MouseUp += value; }
            remove { this.MouseUp -= value; }
        }

        event KeyDownEventHandler IControl.KeyDown
        {
            add { this.KeyDown += value; }
            remove { this.KeyDown -= value; }
        }

        event KeyUpEventHandler IControl.KeyUp
        {
            add { this.KeyUp += value; }
            remove { this.KeyUp -= value; }
        }

        event KeyClickEventHandler IControl.KeyClick
        {
            add { this.KeyClick += value; }
            remove { this.KeyClick -= value; }
        }

        event WidthEventHandler IControl.WidthChanged
        {
            add { this.WidthChanged += value; }
            remove { this.WidthChanged -= value; }
        }

        event HeightEventHandler IControl.HeightChanged
        {
            add { this.HeightChanged += value; }
            remove { this.HeightChanged -= value; }
        }

        event BoundsEventHandler IControl.BoundsChanged
        {
            add { this.BoundsChanged += value; }
            remove { this.BoundsChanged -= value; }
        }

        #endregion
    }

}
