//using CommonEventArgs;
using System;
using Microsoft;
//using System.Windows.Forms;
using System.Collections;
//using System.Drawing;
//using Microsoft.VisualBasic;
//using System.Windows;
//using System.Diagnostics;
//using System.Data;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace xFrippery
{
	public class GUIReader 
	{
                     
        protected internal ControlCollection mobjControls = new ControlCollection();

        public ControlCollection Controls
		{
			get
			{
				return mobjControls;
			}
			set
			{
				mobjControls = value;
			}
		}

        public ControlCollection Load(System.IO.Stream stream) 
        {
            ControlCollection Tmp;
            try
            {
                Tmp = FromStream(stream);
            }
            catch (Exception)
            {
                return null;
            }
            return Tmp;
        }

        public ControlCollection Load(string filename)
		{
            ControlCollection Tmp;
			try
			{
				Tmp = FromFile(filename);
			}
			catch (Exception)
			{
				return null;
			}
			return Tmp;
		}

        private ControlCollection FromStream(System.IO.Stream stream)
		{
			System.Xml.XmlDocument Doc = new System.Xml.XmlDocument();
			Doc.Load(stream);
			ControlCollection Items = new ControlCollection();
			if (Doc.DocumentElement.Name.ToLower().ToString() != "xfrippery")
			{
                throw new System.IO.InvalidDataException("File does not contain a root elemnet named 'xfrippery'.");
			}

            // load controls
            Items = LoadControls(Doc.DocumentElement);
            Doc = null;
			return Items;
		}

        private ControlCollection FromFile(string filename)
		{
			if (filename == string.Empty)
			{
				return null;
			}
			if (System.IO.File.Exists(filename) == false)
			{
				return null;
			}
			System.IO.StreamReader F = new System.IO.StreamReader(filename);
			ControlCollection Items;
			Items = FromStream(F.BaseStream);
			F.Close();
			F = null;
			return Items;
		}
              
       private ControlCollection LoadControls(System.Xml.XmlNode node)
		{
            ControlCollection Items = new ControlCollection();
			foreach (System.Xml.XmlNode nx in node.ChildNodes)
			{
				switch (nx.Name.ToLower().Trim())
				{
					case "control":
						
						Control C;
						C = LoadControl(nx);
						if (C != null)
						{
							Items.Add(C);
						}
						break;
				}
			}
			return Items;
		}

        private Control LoadControl(System.Xml.XmlNode Node)
		{
			Control Item = new Control();
			
			// get name
			Item.name = ReadAttribute(Node, "name", string.Empty);
			// get units
			//string Units;
			//Units = ReadAttribute(Node, "units", "");
			
			// process propseties
			foreach (System.Xml.XmlNode nx in Node.ChildNodes)
			{
				switch (nx.Name.ToLower().Trim())
				{
					case "anchor":
						
						string anchorVal;
                        AnchorStyles T = AnchorStyles.None;
						AnchorStyles L = AnchorStyles.None;
						AnchorStyles R = AnchorStyles.None;
						AnchorStyles B = AnchorStyles.None;
						anchorVal = ReadAttribute(nx, "top", "true").Trim().ToLower();
						if (anchorVal == "true")
						{
							T = AnchorStyles.Top;
						}
						else
						{
							T = AnchorStyles.None;
						}
						anchorVal = ReadAttribute(nx, "left", "true").Trim().ToLower();
						if (anchorVal == "true")
						{
							L = AnchorStyles.Left;
						}
						else
						{
							L = AnchorStyles.None;
						}
						anchorVal = ReadAttribute(nx, "right", "false").Trim().ToLower();
						if (anchorVal == "true")
						{
							R = AnchorStyles.Right;
						}
						else
						{
							R = AnchorStyles.None;
						}
						anchorVal = ReadAttribute(nx, "bottom", "false").Trim().ToLower();
						if (anchorVal == "true")
						{
							B = AnchorStyles.Bottom;
						}
						else
						{
							B = AnchorStyles.None;
						}

                        Item.Anchor = T | L | R | B;
						break;
						
					case "bounds":

                        string boundsVal;
                        boundsVal = ReadAttribute(nx, "left", "0").Trim().ToLower();
                        if (!int.TryParse(boundsVal, out Item.bounds.X))
                        {
                            throw new System.IO.InvalidDataException("Bounds Left contained invalid data.");
                        }
                        boundsVal = ReadAttribute(nx, "top", "0").Trim().ToLower();
                        if (!int.TryParse(boundsVal, out Item.bounds.Y))
                        {
                            throw new System.IO.InvalidDataException("Bounds Top contained invalid data.");
                        }
                        boundsVal = ReadAttribute(nx, "width", "0").Trim().ToLower();
                        if (!int.TryParse(boundsVal, out Item.bounds.Width))
                        {
                            throw new System.IO.InvalidDataException("Bounds Width contained invalid data.");
                        }
                        boundsVal = ReadAttribute(nx, "height", "0").Trim().ToLower();
                        if (!int.TryParse(boundsVal, out Item.bounds.Height))
                        {
                            throw new System.IO.InvalidDataException("Bounds Height contained invalid data.");
                        }
                        break;

                    case "left":
                        if (!int.TryParse(nx.InnerText, out Item.bounds.X))
                        {
                            throw new System.IO.InvalidDataException("'left' element contained invalid data.");
                        }
                        break;

                    case "top":
                        if (!int.TryParse(nx.InnerText, out Item.bounds.Y))
                        {
                            throw new System.IO.InvalidDataException("'top' element contained invalid data.");
                        }
                        break;

                    case "width":
                        if (!int.TryParse(nx.InnerText, out Item.bounds.Width))
                        {
                            throw new System.IO.InvalidDataException("'width' element contained invalid data.");
                        }
                        break;

                    case "height":
                        if (!int.TryParse(nx.InnerText, out Item.bounds.Height))
                        {
                            throw new System.IO.InvalidDataException("'height' element contained invalid data.");
                        }
                        break;

					case "data": 						
						Item.Data = nx.InnerXml;
						break;
						
					case "control":
						
						Control Ctrl;
						Ctrl = LoadControl(nx);
                        Ctrl.Parent = Item;
						Item.Children.Add(Ctrl);
						break;
				}
			}
			
			return Item;
		}

        internal string ReadAttribute(System.Xml.XmlNode NX, string Name, string DefaultValue)
		{
			if (NX == null)
			{
				return DefaultValue;
			}
			foreach (System.Xml.XmlAttribute att in NX.Attributes)
			{
				if (att.Name.Trim().ToLower() == Name.Trim().ToLower())
				{
					return att.Value;
				}
			}
			return DefaultValue;
		}
		
	}
	
}
