//using CommonEventArgs;
using System;
using Microsoft;
//using System.Windows.Forms;
using System.Collections;
//using System.Drawing;
//using Microsoft.VisualBasic;
//using System.Windows;
//using System.Diagnostics;
//using System.Data;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace xFrippery
{
    public static class GUIReader
    {

        public static List<T> Load<T>(System.IO.Stream stream) where T : IControl, new()
        {

            List<T> Tmp;
            try
            {
                Tmp = FromStream<T>(stream);
            }
            catch (Exception)
            {
                return null;
            }
            return Tmp;
        }

        public static List<T> Load<T>(string filename) where T : IControl, new()
        {
            List<T> Tmp;
            try
            {
                Tmp = FromFile<T>(filename);
            }
            catch (Exception)
            {
                return null;
            }
            return Tmp;
        }

        internal static List<T> FromStream<T>(System.IO.Stream stream) where T : IControl, new()
        {
            System.Xml.XmlDocument Doc = new System.Xml.XmlDocument();
            Doc.Load(stream);
            List<T> Items = new List<T>();
            if (Doc.DocumentElement.Name.ToLower().ToString() != "xfrippery")
            {
                throw new System.IO.InvalidDataException("File does not contain a root elemnet named 'xfrippery'.");
            }

            // load controls
            Items = LoadControls<T>(Doc.DocumentElement);
            Doc = null;
            return Items;
        }

        internal static List<T> FromFile<T>(string filename) where T : IControl, new()
        {
            if (filename == string.Empty)
            {
                return null;
            }
            if (System.IO.File.Exists(filename) == false)
            {
                return null;
            }
            System.IO.StreamReader F = new System.IO.StreamReader(filename);
            List<T> Items;
            Items = FromStream<T>(F.BaseStream);
            F.Close();
            F = null;
            return Items;
        }

        internal static List<T> LoadControls<T>(System.Xml.XmlNode node) where T : IControl, new()
        {
            List<T> Items = new List<T>();
            foreach (System.Xml.XmlNode nx in node.ChildNodes)
            {
                switch (nx.Name.ToLower().Trim())
                {
                    case "control":

                        T ctrl;
                        ctrl = LoadControl<T>(nx);
                        if (ctrl != null)
                        {
                            Items.Add(ctrl);
                        }
                        break;
                }
            }
            return Items;
        }

        //internal static ControlCollection<Control> LoadControls(System.Xml.XmlNode node)
        //{
        //    ControlCollection<IControl> Items = new ControlCollection<IControl>();
        //    foreach (System.Xml.XmlNode nx in node.ChildNodes)
        //    {
        //        switch (nx.Name.ToLower().Trim())
        //        {
        //            case "control":

        //                Control C;
        //                C = LoadControl(nx);
        //                if (C != null)
        //                {
        //                    Items.Add(C);
        //                }
        //                break;
        //        }
        //    }
        //    return Items;
        //}

        internal static T LoadControl<T>(System.Xml.XmlNode Node) where T : IControl, new()
        {
            T Item = new T();

            // get name            
            Item.Name = ReadAttribute(Node, "name", string.Empty);
           
            Rectangle rect = Rectangle.Empty;
            // process propseties
            foreach (System.Xml.XmlNode nx in Node.ChildNodes)
            {
                switch (nx.Name.ToLower().Trim())
                {
                    case "anchor":

                        string anchorVal;
                        AnchorStyles tStyle = AnchorStyles.None;
                        AnchorStyles lStyle = AnchorStyles.None;
                        AnchorStyles rStyle = AnchorStyles.None;
                        AnchorStyles bStyle = AnchorStyles.None;
                        anchorVal = ReadAttribute(nx, "top", "true").Trim().ToLower();
                        tStyle = (anchorVal == "true") ? AnchorStyles.Top : AnchorStyles.None;
                      
                        anchorVal = ReadAttribute(nx, "left", "true").Trim().ToLower();
                        lStyle = (anchorVal == "true") ? AnchorStyles.Left : AnchorStyles.None;
                     
                        anchorVal = ReadAttribute(nx, "right", "false").Trim().ToLower();
                        rStyle = (anchorVal == "true") ? AnchorStyles.Right : AnchorStyles.None;
                     
                        anchorVal = ReadAttribute(nx, "bottom", "false").Trim().ToLower();
                        bStyle = (anchorVal == "true") ? AnchorStyles.Bottom : AnchorStyles.None;

                        Item.Anchor = tStyle | lStyle | rStyle | bStyle;
                        break;

                    case "bounds":

                        string boundsVal;
                        boundsVal = ReadAttribute(nx, "left", "0").Trim().ToLower();
                        if (!int.TryParse(boundsVal, out rect.X))
                        {
                            throw new System.IO.InvalidDataException("Bounds Left contained invalid data.");
                        }
                        boundsVal = ReadAttribute(nx, "top", "0").Trim().ToLower();
                        if (!int.TryParse(boundsVal, out rect.Y))
                        {
                            throw new System.IO.InvalidDataException("Bounds Top contained invalid data.");
                        }
                        boundsVal = ReadAttribute(nx, "width", "0").Trim().ToLower();
                        if (!int.TryParse(boundsVal, out rect.Width))
                        {
                            throw new System.IO.InvalidDataException("Bounds Width contained invalid data.");
                        }
                        boundsVal = ReadAttribute(nx, "height", "0").Trim().ToLower();
                        if (!int.TryParse(boundsVal, out rect.Height))
                        {
                            throw new System.IO.InvalidDataException("Bounds Height contained invalid data.");
                        }
                        break;

                    case "left":
                        if (!int.TryParse(nx.InnerText, out rect.X))
                        {
                            throw new System.IO.InvalidDataException("'left' element contained invalid data.");
                        }
                        break;

                    case "top":
                        if (!int.TryParse(nx.InnerText, out rect.Y))
                        {
                            throw new System.IO.InvalidDataException("'top' element contained invalid data.");
                        }
                        break;

                    case "width":
                        if (!int.TryParse(nx.InnerText, out rect.Width))
                        {
                            throw new System.IO.InvalidDataException("'width' element contained invalid data.");
                        }
                        break;

                    case "height":
                        if (!int.TryParse(nx.InnerText, out rect.Height))
                        {
                            throw new System.IO.InvalidDataException("'height' element contained invalid data.");
                        }
                        break;

                    case "data":
                        Item.Data = nx.InnerXml;
                        break;

                }
            }

            // Set control bounds
            Item.Bounds = rect;
           
            // process child controls
            foreach (System.Xml.XmlNode nx in Node.ChildNodes)
            {
                switch (nx.Name.ToLower().Trim())
                {
                    case "control":

                        T Ctrl;
                        Ctrl = LoadControl<T>(nx);
                        Ctrl.Parent = Item;
                        Item.Children.Add(Ctrl);
                        break;
                }

            }


            return Item;
        }

        internal static string ReadAttribute(System.Xml.XmlNode NX, string Name, string DefaultValue)
        {
            if (NX == null)
            {
                return DefaultValue;
            }
            foreach (System.Xml.XmlAttribute att in NX.Attributes)
            {
                if (att.Name.Trim().ToLower() == Name.Trim().ToLower())
                {
                    return att.Value;
                }
            }
            return DefaultValue;
        }
    }

}
